@@ -15,7 +15,7 @@ import urlparse
 from request import Server, Request
 from response import Response
 from router import Router
-import routes
+import routes as default_routes
 from utils import HTTPException
 
 
@@ -91,18 +91,25 @@ class RequestRewriter(object):
         :param request_handler: BaseHTTPRequestHandler for which to
                                 rewrite the request.
         """
-        if request_handler.path in self.rules:
-            methods, destination = self.rules[request_handler.path]
+        split_url = urlparse.urlsplit(request_handler.path)
+        if split_url.path in self.rules:
+            methods, destination = self.rules[split_url.path]
             if "*" in methods or request_handler.command in methods:
                 logger.debug("Rewriting request path %s to %s" %
                              (request_handler.path, destination))
-                request_handler.path = destination
+                new_url = list(split_url)
+                new_url[2] = destination
+                new_url = urlparse.urlunsplit(new_url)
+                request_handler.path = new_url
 
 
 class WebTestServer(ThreadingMixIn, BaseHTTPServer.HTTPServer):
     allow_reuse_address = True
     acceptable_errors = (errno.EPIPE, errno.ECONNABORTED)
 
+    # Ensure that we don't hang on shutdown waiting for requests
+    daemon_threads = True
+
     def __init__(self, server_address, RequestHandlerClass, router, rewriter, bind_hostname, config=None,
                  use_ssl=False, certificate=None, **kwargs):
         """Server for HTTP(s) Requests
@@ -233,7 +240,7 @@ class WebTestRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
 
         except socket.timeout, e:
             self.log_error("Request timed out: %r", e)
-            self.close_connection = 1
+            self.close_connection = True
             return
 
         except Exception as e:
@@ -244,14 +251,18 @@ class WebTestRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             logger.error(err)
 
     def get_request_line(self):
-        self.raw_requestline = self.rfile.readline(65537)
+        try:
+            self.raw_requestline = self.rfile.readline(65537)
+        except socket.error:
+            self.close_connection = True
+            return False
         if len(self.raw_requestline) > 65536:
                 self.requestline = ''
                 self.request_version = ''
                 self.command = ''
                 return False
         if not self.raw_requestline:
-            self.close_connection = 1
+            self.close_connection = True
         return True
 
 
@@ -270,20 +281,48 @@ class WebTestHttpd(object):
     :param rewrites: List of rewrites with which to initialize the rewriter_cls
     :param config: Dictionary holding environment configuration settings for
                    handlers to read, or None to use the default values.
-    :param bind_hostname Boolean indicating whether to bind server to hostname.
+    :param bind_hostname: Boolean indicating whether to bind server to hostname.
 
     HTTP server designed for testing scenarios.
 
     Takes a router class which provides one method get_handler which takes a Request
-    and returns a handler function."
+    and returns a handler function.
+
+    .. attribute:: host
+
+      The host name or ip address of the server
+
+    .. attribute:: port
+
+      The port on which the server is running
+
+    .. attribute:: router
+
+      The Router object used to associate requests with resources for this server
+
+    .. attribute:: rewriter
+
+      The Rewriter object used for URL rewriting
+
+    .. attribute:: use_ssl
+
+      Boolean indicating whether the server is using ssl
+
+    .. attribute:: started
+
+      Boolean indictaing whether the server is running
+
     """
     def __init__(self, host="127.0.0.1", port=8000,
                  server_cls=None, handler_cls=WebTestRequestHandler,
                  use_ssl=False, certificate=None, router_cls=Router,
-                 doc_root=os.curdir, routes=routes.routes,
+                 doc_root=os.curdir, routes=None,
                  rewriter_cls=RequestRewriter, bind_hostname=True, rewrites=None,
                  config=None):
 
+        if routes is None:
+            routes = default_routes.routes
+
         self.host = host
 
         self.router = router_cls(doc_root, routes)
@@ -297,17 +336,21 @@ class WebTestHttpd(object):
         if use_ssl:
             assert certificate is not None and os.path.exists(certificate)
 
-        self.httpd = server_cls((host, port),
-                                handler_cls,
-                                self.router,
-                                self.rewriter,
-                                config=config,
-                                bind_hostname=bind_hostname,
-                                use_ssl=use_ssl,
-                                certificate=certificate)
-        self.started = False
+        try:
+            self.httpd = server_cls((host, port),
+                                    handler_cls,
+                                    self.router,
+                                    self.rewriter,
+                                    config=config,
+                                    bind_hostname=bind_hostname,
+                                    use_ssl=use_ssl,
+                                    certificate=certificate)
+            self.started = False
 
-        _host, self.port = self.httpd.socket.getsockname()
+            _host, self.port = self.httpd.socket.getsockname()
+        except Exception:
+            logger.error('Init failed! You may need to modify your hosts file. Refer to README.md.');
+            raise
 
     def start(self, block=False):
         """Start the server.
